//********************************************************************/
//* Copyright (C) 2005-2011                                          */
//* MEL O'CAT  X178G243 (at) yahoo (dot) com                         */
//* License terms: GNU General Public License Version 2              */
//*                or any later version                              */
//********************************************************************/
//*4567890123456 (71-character line to adjust editor window) 23456789*/


/**
 *  GMFFBoss.java  0.01 11/01/2011
 *
 *  Version 0.01:
 *  Nov-01-2011: new.
 */

package mmj.util;

import  java.util.*;
import  mmj.lang.*;
import  mmj.gmff.*;
import  mmj.pa.ProofAsst;

public class GMFFBoss extends Boss {

    protected GMFFManager    gmffManager;

    /**
     *  Constructor with BatchFramework for access to environment.
     *
     *  @param batchFramework for access to environment.
     */
    public GMFFBoss(BatchFramework batchFramework) {
        super(batchFramework);
    }

    /**
     *  Executes a single command from the RunParmFile.
     *
     *  @param runParm the RunParmFile line to execute.
     */
    public boolean doRunParmCommand(
                            RunParmArrayEntry runParm)
                        throws IllegalArgumentException,
                               GMFFException {

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_CLEAR)
            == 0) {
            gmffManager          = null;
            return false; // not "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_LOAD_FILE)
            == 0) {
			// don't need to do anything here because
			// LogicalSystemBoss executes
			// gmffManager.forceReinitialization() --
			// we will null out the cached gmffManager
			// just to be safe though.
			gmffManager          = null;

            return false; // not "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_GMFF_INITIALIZE)
            == 0) {
            doGMFFInitialize(runParm);
            return true; // "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_GMFF_EXPORT_PARMS)
            == 0) {
            doRunParmGMFFExportParms(runParm);
            return true; // "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_GMFF_USER_EXPORT_CHOICE)
            == 0) {
            doRunParmGMFFUserExportChoice(runParm);
            return true; // "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_GMFF_USER_TEXT_ESCAPES)
            == 0) {
            doRunParmGMFFUserTextEscapes(runParm);
            return true; // "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_GMFF_PARSE_METAMATH_TYPESET_COMMENT)
            == 0) {
            doGMFFParseMetamathTypesetComment(runParm);
            return true; // "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_GMFF_EXPORT_FROM_FOLDER)
            == 0) {
            doGMFFExportFromFolder(runParm);
            return true; // "consumed"
        }

        if (runParm.name.compareToIgnoreCase(
            UtilConstants.RUNPARM_GMFF_EXPORT_THEOREM)
            == 0) {
            doGMFFExportTheorem(runParm);
            return true; // "consumed"
        }

        return false;
    }

    /**
     *  Fetch a gmffManagerobject, building it if necessary
     *  from previously input RunParms.
     *  <p>
     *  NOTE: The returned gmffManageris "ready to go" but
     *        may not have been "initialized", which means
     *        gmffManagervalidation, etc. The reason that
     *        gmffManageris not initialized here is that
     *        a previous attempt to "initialize" may have
     *        failed due to gmffManagererrors, so to re-do
     *        it here would result in doubled-up error
     *        messages. The Initialize gmffManagerRunParm
     *        Command should be used prior to PrintSyntaxDetails
     *        if a "load and print syntax" is desired.
     *
     *  @return gmffManagerobject, ready to go.
     */
    public GMFFManager getGMFFManager() {

        if (gmffManager == null) {
			gmffManager         =
				batchFramework.
					logicalSystemBoss.
						getLogicalSystem().
							getGMFFManager();
		}
        return gmffManager;
    }

    /**
     *  Executes GMFFManager initializeGMFF function and
     *  prints any messages, etc.
     *
     *  @param runParm RunParmFile line.
     */
    public    void doGMFFInitialize(
                       RunParmArrayEntry runParm)
                           throws GMFFException {

		boolean printTypesettingDefinitionsReport
								= false;
		String parmString;
		if (runParm.values.length > 0) {
			parmString          = runParm.values[0].trim();
			if (parmString.length() > 0) {
				if (parmString.compareToIgnoreCase(
						 GMFFConstants.PRINT_TYPESETTING_DEFINITIONS)
					  	== 0) {
					printTypesettingDefinitionsReport
		                        = true;
				}
				else {
					throw new GMFFException(
						GMFFConstants.ERRMSG_GMFF_INITIALIZE_PARM_0_ERR_1
						+ runParm.values[0]);
				}
			}
		}

		try {
			getGMFFManager().
				gmffInitialize(
					printTypesettingDefinitionsReport);
		}
		catch (GMFFException e) {
			batchFramework.outputBoss.getMessages().
				accumErrorMessage(e.getMessage());
		}

       	batchFramework.outputBoss.printAndClearMessages();

		if (!getGMFFManager().isGMFFInitialized()) {
			throw new GMFFException(
				UtilConstants.ERRMSG_GMFF_INITIALIZATION_ERROR_1);
		}

        return;
     }

	public void doRunParmGMFFExportParms(
                       RunParmArrayEntry runParm)
                           throws GMFFException {

        editRunParmValuesLength(
                 runParm,
                 UtilConstants.
                 	RUNPARM_GMFF_EXPORT_PARMS,
                 8);

		String  exportType  	= runParm.values[0].trim();
		String  onoff       	= runParm.values[1].trim();
		String  typesetDefKeyword
								= runParm.values[2].trim();
		String  exportDirectory
								= runParm.values[3].trim();
		String  exportFileType
								= runParm.values[4].trim();
		String  modelsDirectory
								= runParm.values[5].trim();
		String  modelId     	= runParm.values[6].trim();
		String  charsetEncoding = runParm.values[7].trim();

		String  outputFileName  = null;
		if (runParm.values.length > 8) {
			outputFileName  = runParm.values[8].trim();
			if (outputFileName.length() == 0) {
				outputFileName  = null;
			}
		}

		GMFFExportParms gmffExportParms
								=
			new GMFFExportParms(
					exportType,
					onoff,
					typesetDefKeyword,
					exportDirectory,
					exportFileType,
					modelsDirectory,
					modelId,
					charsetEncoding,
					outputFileName);

		getGMFFManager().
			accumInputGMFFExportParms(
				gmffExportParms);

        batchFramework.outputBoss.printAndClearMessages();
        return;
	}

	public void doRunParmGMFFUserTextEscapes(
                       RunParmArrayEntry runParm)
                           throws GMFFException {

        editRunParmValuesLength(
                 runParm,
                 UtilConstants.
                 	RUNPARM_GMFF_USER_TEXT_ESCAPES,
                 3);

		String  exportType  	= runParm.values[0].trim();

		ArrayList<EscapePair> list
		                        =
			new ArrayList<EscapePair>(
				GMFFConstants.DEFAULT_USER_TEXT_ESCAPES.length);

		int    num;
		String repl;
		int    i                = 1;
		do {
			num                 =
				editRunParmValueInteger(
					runParm.values[i++].trim(),
					GMFFConstants.ERRMSG_CAPTION_ESCAPE_PAIR_NUM);

			if (i < runParm.values.length) {
				repl            = runParm.values[i++].trim();
			}
			else {
				repl            = null;
			}

			EscapePair p        = new EscapePair(num,
				                                 repl);
			list.add(p);

		} while (i < runParm.values.length);

		GMFFUserTextEscapes gmffUserTextEscapes
								=
			new GMFFUserTextEscapes(
					exportType,
					list);

		getGMFFManager().
			accumInputGMFFUserTextEscapesList(
				gmffUserTextEscapes);

        batchFramework.outputBoss.printAndClearMessages();
        return;
	}

	public void doRunParmGMFFUserExportChoice(
                       RunParmArrayEntry runParm)
                           throws GMFFException {

        editRunParmValuesLength(
                 runParm,
                 UtilConstants.
                 	RUNPARM_GMFF_USER_EXPORT_CHOICE,
                 1);

		String  exportTypeOrAll = runParm.values[0].trim();

		GMFFUserExportChoice gmffUserExportChoice
		                        =
			new GMFFUserExportChoice(
					exportTypeOrAll);

		getGMFFManager().
			setInputGMFFUserExportChoice(
				gmffUserExportChoice);

        batchFramework.outputBoss.printAndClearMessages();
        return;
	}

	public void doGMFFParseMetamathTypesetComment(
					RunParmArrayEntry runParm)
						throws IllegalArgumentException {

		boolean runParmPrintOption
								= false;

        editRunParmValuesLength(
                 runParm,
                 UtilConstants.
                 	RUNPARM_GMFF_PARSE_METAMATH_TYPESET_COMMENT,
                 3);
		try {

			if (runParm.values.length > 3) {
				String parmString
				                = runParm.values[3].trim();
				if (parmString.length() > 0) {
					if (parmString.compareToIgnoreCase(
							 GMFFConstants.RUNPARM_PRINT_OPTION)
							== 0) {
						runParmPrintOption
								= true;
					}
					else {
						throw new GMFFException(
							GMFFConstants.ERRMSG_GMFF_PARSE_RUNPARM_PARM_4_ERR_1
							+ runParm.values[3]);
					}
				}
			}

			String typesetDefKeyword
								= runParm.values[0].trim();
			String myDirectory  = runParm.values[1].trim();
			String myMetamathTypesetCommentFileName
								= runParm.values[2].trim();

			getGMFFManager().
				parseMetamathTypesetComment(
					typesetDefKeyword,
					myDirectory,
					myMetamathTypesetCommentFileName,
					runParmPrintOption);
		}
		catch (GMFFException e) {
            batchFramework.
            	outputBoss.
            		getMessages().
						accumErrorMessage(
							e.getMessage());
		}

        batchFramework.outputBoss.printAndClearMessages();
        return;
	}

	public void doGMFFExportFromFolder(
					RunParmArrayEntry runParm)
						throws IllegalArgumentException {

        editRunParmValuesLength(
                 runParm,
                 UtilConstants.
                 	RUNPARM_GMFF_EXPORT_FROM_FOLDER,
                 4);
		try {

			String inputDirectory
			                    = runParm.values[0].trim();
			String theoremLabelOrAsterisk
								= runParm.values[1].trim();
			String inputFileType
								= runParm.values[2].trim();

			String maxNumberToExport
								= runParm.values[3].trim();

			String appendFileName
			                    = null;
			if (runParm.values.length > 4) {
				appendFileName  = runParm.values[4].trim();
				if (appendFileName.length() == 0) {
					appendFileName
					            = null;
				}
			}

			getGMFFManager().
				exportFromFolder(
					inputDirectory,
					theoremLabelOrAsterisk,
					inputFileType,
					maxNumberToExport,
					appendFileName);
		}
		catch (GMFFException e) {
            batchFramework.
            	outputBoss.
            		getMessages().
						accumErrorMessage(
							e.getMessage());
		}

        batchFramework.outputBoss.printAndClearMessages();
        return;
	}

	public void doGMFFExportTheorem(
					RunParmArrayEntry runParm)
						throws IllegalArgumentException {

        editRunParmValuesLength(
                 runParm,
                 UtilConstants.
                 	RUNPARM_GMFF_EXPORT_THEOREM,
                 2);
		try {

			String theoremLabelOrAsterisk
								= runParm.values[0].trim();

			String maxNumberToExport
								= runParm.values[1].trim();

			String appendFileName
			                    = null;
			if (runParm.values.length > 2) {
				appendFileName  = runParm.values[2].trim();
				if (appendFileName.length() == 0) {
					appendFileName
					            = null;
				}
			}

	        ProofAsst proofAsst =
            	batchFramework.proofAsstBoss.getProofAsst();
        	if (proofAsst == null) {
            	throw new GMFFException(
                	UtilConstants.
                    	ERRMSG_GMFF_PROOF_ASST_MISSING_1
                    	+ UtilConstants.
                    		RUNPARM_GMFF_EXPORT_THEOREM
                    	+ UtilConstants.
                    		ERRMSG_GMFF_PROOF_ASST_MISSING_2);
			}

			getGMFFManager().
				exportTheorem(
					theoremLabelOrAsterisk,
					maxNumberToExport,
					appendFileName,
					proofAsst);
		}
		catch (Exception e) {
            batchFramework.
            	outputBoss.
            		getMessages().
						accumErrorMessage(
							e.getMessage());
            batchFramework.
            	outputBoss.
            		getMessages().
						accumErrorMessage(
		                	UtilConstants.
		                    	ERRMSG_GMFF_RUNPARM_ERROR_1
		                    	+ UtilConstants.
			                   		RUNPARM_GMFF_EXPORT_THEOREM
			                   	+ UtilConstants.
	                    			ERRMSG_GMFF_RUNPARM_ERROR_2);
		}

        batchFramework.outputBoss.printAndClearMessages();
        return;
	}
}
